/*
 * Ensys srl
 * 
 * Questo esempio riceve dati via rs485, li visualizza e li replica in trasmissione
 * Setting 115200 8N1
 * 
 * Verificare di aver abilitato la porta UART1, ad esempio integrando nel file
 * 	/boot/config.txt 
 * le seguenti righe:
 * 	#uart1 - rs485/rs232
 * 	dtoverlay=uart1,txd1_pin=32,rxd1_pin=33
 * 	core_freq=250
 * 
 * Per compilare questo file utilizzare il comando:
 * 	gcc -o test uart.c
 * Poi eseguire:
 * 	sudo ./test
 */
 
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>



int main()
{
	int32_t fd_gpio;
	int32_t fd_uart;
        struct termios tty;
	uint8_t buf [100];
	uint32_t byte_rxed_ist = 0u;
	uint32_t byte_rxed_tot = 0u;
	uint8_t loop_wait = 0u;
	
	fd_uart = open("/dev/ttyS0", O_RDWR | O_NOCTTY | O_SYNC);

        tcgetattr(fd_uart, &tty);

        cfsetospeed(&tty, B115200);
        cfsetispeed(&tty, B115200);

        tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;
        tty.c_cflag |= (CLOCAL | CREAD);
        tty.c_cflag &= ~(PARENB | PARODD);
        tty.c_cflag &= ~CSTOPB;
        tty.c_cflag &= ~CRTSCTS;

        tty.c_iflag &= ~IGNBRK;
        tty.c_iflag &= ~(IXON | IXOFF | IXANY);

        tty.c_lflag = 0;

        tty.c_oflag = 0;

        tty.c_cc[VMIN]  = 0;
        tty.c_cc[VTIME] = 1;

        tcsetattr(fd_uart, TCSANOW, &tty);

	fd_gpio = open("/sys/class/gpio/export", O_WRONLY|O_SYNC);
	write(fd_gpio, "31", sizeof("31"));
	close(fd_gpio);

	fd_gpio = open("/sys/class/gpio/gpio31/direction", O_WRONLY|O_SYNC);
	write(fd_gpio, "out", sizeof("out"));
	close(fd_gpio);

	fd_gpio = open("/sys/class/gpio/gpio31/value", O_WRONLY|O_SYNC);
	write(fd_gpio, "0", sizeof("0"));
	close(fd_gpio);


	while(1)
	{

		byte_rxed_ist = read (fd_uart, &buf[byte_rxed_tot], sizeof(buf)-byte_rxed_tot);
		if(byte_rxed_ist>0){
			byte_rxed_tot += byte_rxed_ist;
			loop_wait = 0u;
		}
		else{
			if(byte_rxed_tot>0u){
				loop_wait++;
			}
		}
		
		if(loop_wait>10){
			printf("rxed: ");
			for(int i=0;i<byte_rxed_tot;i++){
				printf("%02x ", buf[i]);
			}
			printf("\n");
			
			fd_gpio = open("/sys/class/gpio/gpio31/value", O_WRONLY|O_SYNC);
			write(fd_gpio, "1", sizeof("1"));
			close(fd_gpio);

			write(fd_uart, buf, byte_rxed_tot);
			tcdrain(fd_uart);

			fd_gpio = open("/sys/class/gpio/gpio31/value", O_WRONLY|O_SYNC);
			write(fd_gpio, "0", sizeof("0"));
			close(fd_gpio);

			byte_rxed_tot = 0u;
			loop_wait = 0u;
		}

	}
	
	return 0;
}
